/*
    This is a romfs file system driver for Windows NT and Windows 2000.
    Copyright (C) 1999, 2000, 2001 Bo Brantn.
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "ntifs.h"
#include "fsd.h"
#include "rom_fs.h"
#include "border.h"

#pragma code_seg(FSD_PAGED_CODE)

//
// When not debugging the fast I/O read entry point is set directly to
// FsRtlCopyRead.
//

#if DBG

BOOLEAN
FsdFastIoRead (
    IN PFILE_OBJECT         FileObject,
    IN PLARGE_INTEGER       FileOffset,
    IN ULONG                Length,
    IN BOOLEAN              Wait,
    IN ULONG                LockKey,
    OUT PVOID               Buffer,
    OUT PIO_STATUS_BLOCK    IoStatus,
    IN PDEVICE_OBJECT       DeviceObject
    )
{
    BOOLEAN     Status;
    PFSD_FCB    Fcb;

    PAGED_CODE();

    Fcb = (PFSD_FCB) FileObject->FsContext;

    ASSERT(Fcb != NULL);

    ASSERT((Fcb->Identifier.Type == FCB) &&
           (Fcb->Identifier.Size == sizeof(FSD_FCB)));

    KdPrint((
        DRIVER_NAME ": %-16.16s %-31s %s Offset: %I64u Length: %u\n",
        PsGetCurrentProcess()->ImageFileName,
        "FASTIO_READ",
        Fcb->DbgFileName,
        FileOffset->QuadPart,
        Length
        ));

    Status = FsRtlCopyRead (
        FileObject,
        FileOffset,
        Length,
        Wait,
        LockKey,
        Buffer,
        IoStatus,
        DeviceObject
        );

    if (Status == FALSE)
    {
        KdPrint((
            DRIVER_NAME ": %-16.16s %-31s *** Status: FALSE ***\n",
            PsGetCurrentProcess()->ImageFileName,
            "FASTIO_READ"
            ));
    }
    else if (IoStatus->Status != STATUS_SUCCESS)
    {
        KdPrint((
            DRIVER_NAME ": %-16.16s %-31s *** Status: %s (%#x) ***\n",
            PsGetCurrentProcess()->ImageFileName,
            "FASTIO_READ",
            FsdNtStatusToString(IoStatus->Status),
            IoStatus->Status
            ));
    }

    return Status;
}

#endif // DBG

BOOLEAN
FsdFastIoQueryBasicInfo (
    IN PFILE_OBJECT             FileObject,
    IN BOOLEAN                  Wait,
    OUT PFILE_BASIC_INFORMATION Buffer,
    OUT PIO_STATUS_BLOCK        IoStatus,
    IN PDEVICE_OBJECT           DeviceObject
    )
{
    PFSD_FCB Fcb;

    PAGED_CODE();

    if (DeviceObject == FsdGlobalData.DeviceObject)
    {
        return FALSE;
    }

    Fcb = (PFSD_FCB) FileObject->FsContext;

    ASSERT(Fcb != NULL);

    if (Fcb->Identifier.Type == VCB)
    {
        return FALSE;
    }

    ASSERT((Fcb->Identifier.Type == FCB) &&
           (Fcb->Identifier.Size == sizeof(FSD_FCB)));

    KdPrint((
        DRIVER_NAME ": %-16.16s %-31s %s\n",
        PsGetCurrentProcess()->ImageFileName,
        "FASTIO_QUERY_BASIC_INFO",
        Fcb->DbgFileName
        ));

    FsRtlEnterFileSystem();

    if (!ExAcquireResourceSharedLite(
            &Fcb->MainResource,
            Wait
            ))
    {
        FsRtlExitFileSystem();
        return FALSE;
    }

    RtlZeroMemory(Buffer, sizeof(FILE_BASIC_INFORMATION));

    Buffer->CreationTime.QuadPart = 0;

    Buffer->LastAccessTime.QuadPart = 0;

    Buffer->LastWriteTime.QuadPart = 0;

    Buffer->ChangeTime.QuadPart = 0;

    Buffer->FileAttributes = Fcb->FileAttributes;

    IoStatus->Information = sizeof(FILE_BASIC_INFORMATION);

    IoStatus->Status = STATUS_SUCCESS;

    ExReleaseResourceForThreadLite(
        &Fcb->MainResource,
        ExGetCurrentResourceThread()
        );

    FsRtlExitFileSystem();

    return TRUE;
}

BOOLEAN
FsdFastIoQueryStandardInfo (
    IN PFILE_OBJECT                 FileObject,
    IN BOOLEAN                      Wait,
    OUT PFILE_STANDARD_INFORMATION  Buffer,
    OUT PIO_STATUS_BLOCK            IoStatus,
    IN PDEVICE_OBJECT               DeviceObject
    )
{
    PFSD_FCB Fcb;

    PAGED_CODE();

    if (DeviceObject == FsdGlobalData.DeviceObject)
    {
        return FALSE;
    }

    Fcb = (PFSD_FCB) FileObject->FsContext;

    ASSERT(Fcb != NULL);

    if (Fcb->Identifier.Type == VCB)
    {
        return FALSE;
    }

    ASSERT((Fcb->Identifier.Type == FCB) &&
           (Fcb->Identifier.Size == sizeof(FSD_FCB)));

    KdPrint((
        DRIVER_NAME ": %-16.16s %-31s %s\n",
        PsGetCurrentProcess()->ImageFileName,
        "FASTIO_QUERY_STANDARD_INFO",
        Fcb->DbgFileName
        ));

    FsRtlEnterFileSystem();

    if (!ExAcquireResourceSharedLite(
            &Fcb->MainResource,
            Wait
            ))
    {
        FsRtlExitFileSystem();
        return FALSE;
    }

    RtlZeroMemory(Buffer, sizeof(FILE_STANDARD_INFORMATION));

    Buffer->AllocationSize.QuadPart = be32_to_cpu(Fcb->romfs_inode->size);

    Buffer->EndOfFile.QuadPart = be32_to_cpu(Fcb->romfs_inode->size);

    Buffer->NumberOfLinks = 1;

#ifndef FSD_RO

    Buffer->DeletePending = Fcb->DeletePending;

#else

    Buffer->DeletePending = FALSE;

#endif

    if (Fcb->FileAttributes & FILE_ATTRIBUTE_DIRECTORY)
    {
        Buffer->Directory = TRUE;
    }
    else
    {
        Buffer->Directory = FALSE;
    }

    IoStatus->Information = sizeof(FILE_STANDARD_INFORMATION);

    IoStatus->Status = STATUS_SUCCESS;

    ExReleaseResourceForThreadLite(
        &Fcb->MainResource,
        ExGetCurrentResourceThread()
    );

    FsRtlExitFileSystem();

    return TRUE;
}

BOOLEAN
FsdFastIoQueryNetworkOpenInfo (
    IN PFILE_OBJECT                     FileObject,
    IN BOOLEAN                          Wait,
    OUT PFILE_NETWORK_OPEN_INFORMATION  Buffer,
    OUT PIO_STATUS_BLOCK                IoStatus,
    IN PDEVICE_OBJECT                   DeviceObject
    )
{
    PFSD_FCB Fcb;

    PAGED_CODE();

    if (DeviceObject == FsdGlobalData.DeviceObject)
    {
        return FALSE;
    }

    Fcb = (PFSD_FCB) FileObject->FsContext;

    ASSERT(Fcb != NULL);

    if (Fcb->Identifier.Type == VCB)
    {
        return FALSE;
    }

    ASSERT((Fcb->Identifier.Type == FCB) &&
           (Fcb->Identifier.Size == sizeof(FSD_FCB)));

    KdPrint((
        DRIVER_NAME ": %-16.16s %-31s %s\n",
        PsGetCurrentProcess()->ImageFileName,
        "FASTIO_QUERY_NETWORK_OPEN_INFO",
        Fcb->DbgFileName
        ));

    FsRtlEnterFileSystem();

    if (!ExAcquireResourceSharedLite(
            &Fcb->MainResource,
            Wait
            ))
    {
        FsRtlExitFileSystem();
        return FALSE;
    }

    RtlZeroMemory(Buffer, sizeof(FILE_NETWORK_OPEN_INFORMATION));

    Buffer->CreationTime.QuadPart = 0;

    Buffer->LastAccessTime.QuadPart = 0;

    Buffer->LastWriteTime.QuadPart = 0;

    Buffer->ChangeTime.QuadPart = 0;

    Buffer->AllocationSize.QuadPart = be32_to_cpu(Fcb->romfs_inode->size);

    Buffer->EndOfFile.QuadPart = be32_to_cpu(Fcb->romfs_inode->size);

    Buffer->FileAttributes = Fcb->FileAttributes;

    IoStatus->Information = sizeof(FILE_NETWORK_OPEN_INFORMATION);

    IoStatus->Status = STATUS_SUCCESS;

    ExReleaseResourceForThreadLite(
        &Fcb->MainResource,
        ExGetCurrentResourceThread()
        );

    FsRtlExitFileSystem();

    return TRUE;
}

#pragma code_seg()  // end FSD_PAGED_CODE
